@@ -4,9 +4,9 @@ use strict;
 use warnings;
 
 use vars qw( $VERSION $AUTOLOAD );
-$VERSION = '1.02';
+$VERSION = '1.03';
 
-use Scalar::Util qw( blessed refaddr reftype );
+use Scalar::Util qw( blessed refaddr reftype weaken );
 use UNIVERSAL::isa;
 use UNIVERSAL::can;
 
@@ -130,7 +130,7 @@ sub remove
 sub called
 {
 	my ($self, $sub) = @_;
-	
+
 	for my $called (reverse @{ _calls( $self ) }) {
 		return 1 if $called->[0] eq $sub;
 	}
@@ -219,15 +219,23 @@ sub dispatch_mocked_method
 		require Carp;
 		Carp::carp("Un-mocked method '$sub()' called");
 	}
+
 	return;
 }
 
 sub log_call
 {
-	my ($self, $sub) = splice( @_, 0, 2 );
-
+	my ($self, $sub, @call_args) = @_;
 	return unless _logs( $self, $sub );
-	push @{ _calls( $self ) }, [ $sub, [ @_ ] ];
+
+	# prevent circular references with weaken
+	for my $arg ( @call_args )
+	{
+		next unless ref $arg;
+		weaken( $arg ) if refaddr( $arg ) eq refaddr( $self );
+	}		
+
+	push @{ _calls( $self ) }, [ $sub, \@call_args ];
 }
 
 sub called_ok
@@ -273,7 +281,7 @@ sub fake_module
 	local $SIG{__WARN__} = sub { $warn->( $_[0] ) unless $_[0] =~ /redefined/ };
 	no strict 'refs';
 	${ $modname . '::' }{VERSION} ||= -1;
-	
+
 	for my $sub (keys %subs)
 	{
 		my $type = reftype( $subs{ $sub } ) || '';
@@ -293,6 +301,15 @@ sub fake_new
 	$self->fake_module( $class, new => sub { $self } );
 }
 
+sub DESTROY
+{
+	my $self = shift;
+	$self->_clear_calls();
+	$self->_clear_subs();
+	$self->_clear_logs();
+	$self->_clear_isas();
+}
+
 sub _get_key
 {
 	my $invocant = shift;
@@ -306,6 +323,11 @@ sub _get_key
 	{
 		$calls{ _get_key( shift ) } ||= [];
 	}
+
+	sub _clear_calls
+	{
+		delete $calls{ _get_key( shift ) };
+	}
 }
 
 {
@@ -315,6 +337,11 @@ sub _get_key
 	{
 		$subs{ _get_key( shift ) } ||= {};
 	}
+
+	sub _clear_subs
+	{
+		delete $subs{ _get_key( shift ) };
+	}
 }
 
 {
@@ -343,6 +370,11 @@ sub _get_key
 		my ($name) = @_;
 		return exists $logs{$key}{$name};
 	}
+
+	sub _clear_logs
+	{
+		delete $logs{ _get_key( shift ) };
+	}
 }
 
 {
@@ -352,6 +384,11 @@ sub _get_key
 	{
 		$isas{ _get_key( shift ) } ||= {};
 	}
+
+	sub _clear_isas
+	{
+		delete $isas{ _get_key( shift ) };
+	}
 }
 
 1;
